@@ -7,6 +7,7 @@ import json
 import os
 import re
 import pandas as pd
+import time
 
 
 def find_tables_by_column_name(column_name: str, database: Optional[str] = None, verbose: bool = False) -> pd.DataFrame:
@@ -31,12 +32,7 @@ def find_tables_by_column_name(column_name: str, database: Optional[str] = None,
 
     return df_by_column
 
-
-file_path = "table_metadata.json"
-database = "staging"
-# load_dotenv(find_dotenv())
-
-def send_metadata_to_redshift(database: Optional[str] = None, file_path: str = "table_metadata.json"):
+def send_metadata_to_redshift(table_name: str, database: Optional[str] = None, file_path: str = "table_metadata.json"):
     """
     Use this function to send the Table and Column Descriptions to Redshift. There is a standard JSON file which this
     function reads to retrieve all the mandatory descriptions.
@@ -54,60 +50,67 @@ def send_metadata_to_redshift(database: Optional[str] = None, file_path: str = "
 
     source_track_df_list = []
     print("💿 Generating the SQL Query from the JSON...")
-    for table in table_metadata_dict["tables"]:
-        print(f"\t📌 {table['name']}:")
-        for key, value in table['details'].items():
-            if key == "description":
-                if len(value) > 600:
-                    raise OverflowError(f"❌ {key.capitalize()} value for table {table['name']} is too long. "
-                                        "Please, limit yourself to 600 chars max.")
-            else:
-                if len(value) > 200:
-                    raise OverflowError(f"❌ {key.capitalize()} value for table {table['name']} is too long. "
-                                        "Please, limit yourself to 200 chars max.")
+    list_tables = [table["name"] for table in table_metadata_dict["tables"]]
+    try:
+        table_id = list_tables.index(table_name)
+        table = table_metadata_dict["tables"][table_id]
+    except ValueError:
+        print("❌ Exiting the process...")
+        print(f"😰 The chosen table {table_name} is not present on the json file.")
+        raise
 
-        print(f"\t\t🛠 Checking the metadata constraints...")
-        table_description_str = f"Country: {table['details']['country']}\n\n" \
-                                f"Table Schema: {table['details']['table_schema']}\n\n" \
-                                f"Description: {table['details']['description']}\n\n" \
-                                f"Source Name: {table['details']['source_name']}\n\n" \
-                                f"Geographical Coverage: {table['details']['geographical_coverage']}\n\n" \
-                                f"Geographical Granularity: {table['details']['geographical_granularity']}\n\n" \
-                                f"Update Frequency: {table['details']['update_frequency']}\n\n" \
-                                f"Last Update: {table['details']['last_update']}\n\n" \
-                                f"Time Frame Coverage: {table['details']['time_coverage']}\n\n" \
-                                f"Caveats: {table['details']['caveats']}\n\n" \
-                                f"Additional Info: {table['details']['additional_information']}\n\n" \
-                                f"Related OIP Block: {table['details']['related_oip_block']}\n\n" \
-                                f"Update Process: {table['details']['update_process']}\n\n" \
-                                f"Import Format: {table['details']['file_import_format']}\n\n" \
-                                f"Import Separator: {table['details']['file_import_separator']}\n\n" \
-                                f"Table Type: {table['details']['table_type']}"
+    print(f"\t📌 {table['name']}:")
+    for key, value in table['details'].items():
+        if key == "description":
+            if len(value) > 600:
+                raise OverflowError(f"❌ {key.capitalize()} value for table {table['name']} is too long. "
+                                    "Please, limit yourself to 600 chars max.")
+        else:
+            if len(value) > 400:
+                raise OverflowError(f"❌ {key.capitalize()} value for table {table['name']} is too long. "
+                                    "Please, limit yourself to 400 chars max.")
 
-        table_description_sql = f"COMMENT ON table {table['details']['table_schema']}.{table['name']} " \
-                                f"IS '{table_description_str}';"
+    print(f"\t\t🛠 Checking the metadata constraints...")
+    table_description_str = f"Country: {table['details']['country']}\n\n" \
+                            f"Table Schema: {table['details']['table_schema']}\n\n" \
+                            f"Description: {table['details']['description']}\n\n" \
+                            f"Source Name: {table['details']['source_name']}\n\n" \
+                            f"Geographical Coverage: {table['details']['geographical_coverage']}\n\n" \
+                            f"Geographical Granularity: {table['details']['geographical_granularity']}\n\n" \
+                            f"Update Frequency: {table['details']['update_frequency']}\n\n" \
+                            f"Last Update: {table['details']['last_update']}\n\n" \
+                            f"Time Frame Coverage: {table['details']['time_coverage']}\n\n" \
+                            f"Caveats: {table['details']['caveats']}\n\n" \
+                            f"Additional Info: {table['details']['additional_information']}\n\n" \
+                            f"Update Process: {table['details']['update_process']}\n\n" \
+                            f"Import Format: {table['details']['file_import_format']}\n\n" \
+                            f"Import Separator: {table['details']['file_import_separator']}\n\n" \
+                            f"Table Type: {table['details']['table_type']}"
 
-        source_track_df_list.append(
-            pd.DataFrame(
-                data={"table_name": [table['name']],
-                      "source_name": [table['details']['source_name']],
-                      "source_link": [table['details']['source_link']]}
-            ))
+    table_description_sql = f"COMMENT ON table {table['details']['table_schema']}.{table['name']} " \
+                            f"IS '{table_description_str}';"
 
-        columns_description_sql_lst = []
-        for column in table["columns"]:
-            sql = f"COMMENT ON column {table['details']['table_schema']}.{table['name']}.{column['name']} " \
-                  f"IS '{column['description']}';"
-            columns_description_sql_lst.append(sql)
-        columns_description_sql = "\n".join(columns_description_sql_lst)
+    source_track_df_list.append(
+        pd.DataFrame(
+            data={"table_name": [table['name']],
+                  "source_name": [table['details']['source_name']],
+                  "source_link": [table['details']['source_link']]}
+        ))
 
-        print("\t\t📋 Launching Table Description query...")
-        ro.conn.execution_options(isolation_level="AUTOCOMMIT").execute(text(table_description_sql))
+    columns_description_sql_lst = []
+    for column in table["columns"]:
+        sql = f"COMMENT ON column {table['details']['table_schema']}.{table['name']}.{column['name']} " \
+              f"IS '{column['description']}';"
+        columns_description_sql_lst.append(sql)
+    columns_description_sql = "\n".join(columns_description_sql_lst)
 
-        print("\t\t🏛 Launching Columns Description query...")
-        ro.conn.execution_options(isolation_level="AUTOCOMMIT").execute(text(columns_description_sql))
+    print("\t\t📋 Launching Table Description query...")
+    ro.conn.execution_options(isolation_level="AUTOCOMMIT").execute(text(table_description_sql))
 
-        print(f"\t\t🎉 Metadata insertion finished for {table['name']}")
+    print("\t\t🏛 Launching Columns Description query...")
+    ro.conn.execution_options(isolation_level="AUTOCOMMIT").execute(text(columns_description_sql))
+
+    print(f"\t\t🎉 Metadata insertion finished for {table['name']}")
     ro.conn.close()
 
     if database == "oip":
@@ -192,7 +195,18 @@ def send_to_redshift(database: str,
     ro.process_table(bucket=kwargs.get("bucket", so.bucket), sep="|", check=kwargs.get("check", False))
 
     if send_metadata:
-        send_metadata_to_redshift(database=database, file_path=json_path)
+        try:
+            send_metadata_to_redshift(table_name=table, database=database, file_path=json_path)
+        except ValueError:
+            print(f"🔍 Please check if the name is correct or set send_metadata to false.")
+            ro.conn.close()
+            raise
+        except FileNotFoundError:
+            print(f"⁉️ The JSON file containing all metadata was not found. Please, either check if the "
+                  f"file path passed is correct, or set send_metadata to False.")
+            ro.conn.close()
+            raise
+
     ro.conn.close()
 
 
@@ -260,8 +274,6 @@ class RedshiftOperator:
         self.df_std_error = None
         self.s3_bucket = None
         self.ddl_table = None
-        self.generate_DDL()
-
         # Session = sessionmaker(bind=self.engine)
         # self.session = Session()
 
@@ -478,6 +490,9 @@ class RedshiftOperator:
         :param not_include: List of schemas to be excluded if any.
         :return:
         """
+        # Generate the DDL table.
+        self.generate_DDL()
+        time.sleep(2)  # Allow a little time for the query to finish executing.
 
         if not_include is None:
             not_include = [
